
; ASTRO version 1.0
;  Benjamin Moody
;  benjamin@ecg.mit,edu

;------------------------------------------------
;  Drawing Routines
;------------------------------------------------

RedrawScreen:
#ifdef NO_ONKEY
    set indicOnly,(iy+indicFlags
#endif

    B_CALL _RunIndicOn
    ld a,busyNormal
    ld (indicBusy),a
    
    call CalcViewAngles

    B_CALL _GrBufClr
    set bufferOnly,(iy+plotFlag3)
    res plotLoc,(iy+plotFlags)
    set useFastCirc,(iy+plotFlag3)
    set textWrite,(iy+sGrFlags)

    ld a,47
    ld (curGX2),a
    ld (curGX),a
    ld a,32
    ld (curGY2),a
    ld a,63
    ld (curGY),a
    B_CALL _GrphCirc

    ld a,66
    ld (penCol),a
    xor a
    ld (penRow),a
    ld hl,NameLbl
    call VPutS

    ld hl,RoseSpr
    ld de,appBackUpScreen
    ld bc,RoseSprEnd-RoseSpr
    ldir
    ld hl,appBackUpScreen
    ld de,35*256+79
    B_CALL _DisplayImage

    res bufferOnly,(iy+plotFlag3)
    res textWrite,(iy+sGrFlags)
    B_CALL _GrBufCpy
    
    ld a,(NumObjs)
    ld b,a
    ld c,0
    ld hl,ObjectsOn
DrawScreenLoop:
    ld a,(hl)
    or a
    call z,DrawObjectSet
    dec b
    inc c
    inc hl
    ld a,b
    or a
    jr nz,DrawScreenLoop

#ifdef NO_ONKEY
    res indicOnly,(iy+indicFlags)
#endif
    B_CALL _GrBufCpy
    B_CALL _RunIndicOff
    set plotLoc,(iy+plotFlags)
    ret

DrawObjectSet:
    push hl     ; because hl and bc are being used in RedrawScreen
    push bc     ; REMEMBER to pop these at the end of each "draw" function
    
    ld a,c
    or a
    cp 4    ; First 4 sets are built-in, others external
;    jr nc,DrawExtObjSet

;    or a
;    jr z,DrawSun
;    cp 1
;    jr z,DrawMoon
;    cp 2
;    jr z,DrawPlanets
DrawDefStars:
    ld hl,Stars
DrawStarsX:
    ld c,(hl)   ; Minutes RA
    inc hl
    ld b,(hl)   ; Hours RA
    inc hl
    ld a,c
    cp $ff
    jr z,DrStXDone
    cp $fe
    jr z,DrStXPutLbl
    cp $fd
    jr z,DrStXStarName

    push hl
    call HMToFloat
    B_CALL _OP1ToOP4
    pop hl
    ld c,(hl)   ; Minutes Decl.
    inc hl
    ld b,(hl)   ; Degrees Decl.
    inc hl

    ld d,(hl)
    inc hl
    ld a,(StarDepth)
    cp d
    jr c,DrawStarsX

    push hl
    call DMToFloat
    B_CALL _OP4ToOP2
    call CalcStar

    xor a
    call DrawObj
    pop hl
    
#ifndef NO_ONKEY
    bit onInterrupt,(iy+onFlags)
    jr z,DrawStarsX
    res onInterrupt,(iy+onFlags)
    B_CALL _GrBufCpy
    res bufferOnly,(iy+plotFlag3)
    B_CALL _RunIndicOff
    set plotLoc,(iy+plotFlags)
    jp MainLoop
#else
    jr DrawStarsX
#endif

DrStXDone:
    pop bc
    pop hl
    ret

DrStXPutLbl:
    dec hl
    ld bc,0
    ld (penCol),bc
    call VPutS
    ld a,32
    ld b,5
DSXPLLoop:
    B_CALL _VPutMap
    djnz DSXPLLoop
    jr DrawStarsX

DrStXStarName:
    ; Skipping star name while drawing
    ld a,(hl)
    inc hl
    or a
    jr z,DrawStarsX
    jr DrStXStarName

;DrawExtObjSet:
;    ld l,a
;    ld h,9
;    B_CALL _HTimesL
;    ld bc,DataLoc
;    add hl,bc
;    rst rMov9ToOP1
;    B_CALL _ChkFindSym
;    jr c,DrStXDone
;    ld a,b
;    or a
;    jr nz,DrStXDone     ; Reading from flash not yet supported


DrawObj:
    ; OP1:  Az
    ; FPS:  Alt
    push af
                
    B_CALL _OP1ToOP6
    B_CALL _Sin
    ld hl,float31
    call Mov9ToOP2
    B_CALL _FPMult

    B_CALL _OP1ToOP5
    B_CALL _PopRealO1
    B_CALL _CkOP1Pos
    ret nz
    ld hl,float90
    call Mov9ToOP2
    B_CALL _OP2ToOP3
    B_CALL _FPSub
    B_CALL _OP3ToOP2
    B_CALL _FPDiv
    rst rPushRealO1
    B_CALL _OP5ToOP2
    
    B_CALL _FPMult
    ld hl,float48
    call Mov9ToOP2
    rst rFPAdd
    B_CALL _ConvOP1
    ld b,a
    push bc
    B_CALL _OP6ToOP1
    B_CALL _Cos
    ld hl,float31
    call Mov9ToOP2
    B_CALL _FPMult
    B_CALL _PopRealO2
    B_CALL _FPMult
    ld hl,float31
    call Mov9ToOP2
    B_CALL _FPSub
    call FPNegate
    B_CALL _ConvOP1
    pop bc
    ld c,a
    
    pop af      ; get point type       
    cp 1       ; if it's the box shape, don't draw the center pixel
    jr z,DrawObjBox
    B_CALL _PointOn ;   x
    or a
    ret z           ; if single pixel, you're done
DrawObjCross:
    inc b           ; otherwise cross shape
                    ;   o
                    ;  o*o
    B_CALL _PointOn ;   x

    dec b           ;   o
    inc c           ;  o*x
    B_CALL _PointOn ;   *

    dec c           ;   x
    dec b           ;  o**
    B_CALL _PointOn ;   *

    inc b           ;   *
    dec c           ;  x**
    B_CALL _PointOn ;   *

    xor a   ; To indicate that it was drawn.
    ret
DrawObjBox:
            ; box shape
    inc b           ;   ooo
    B_CALL _PointOn ;   o o
                    ;   oxo

    inc c           ;   ooo
    B_CALL _PointOn ;   o o
                    ;   o*x

    dec b           ;   ooo
    B_CALL _PointOn ;   o x
                    ;   o**
        
    dec b           ;   oox
    B_CALL _PointOn ;   o *
                    ;   o**    
    
    dec c           ;   ox*
    B_CALL _PointOn ;   o *
                    ;   o**
    
    dec c           ;   x**
    B_CALL _PointOn ;   o *
                    ;   o**
        
    inc b           ;   ***
    B_CALL _PointOn ;   x *
                    ;   o**
    
    inc b           ;   ***
    B_CALL _PointOn ;   * *
                    ;   x**
    xor a
    ret

VPut2DecNum:
    ld b,'0'
    or a
VPut2DNLp:
    cp 10
    jr c, VPut2DNCnt
    sub 10
    inc b
    jr VPut2DNLp
VPut2DNCnt:
    ld c,a
    ld a,b
    B_CALL _VPutMap
    ld a,c
    ld b,'0'
    add a,b
    B_CALL _VPutMap
    ret

VPut3DecNum:
    ld b,'0'
VPut3DNLp:
    cp 100
    jr c, VPut3DNCnt2
    sub 100
    inc b
    jr VPut3DNLp
VPut3DNCnt2:
    push af
    ld a,b
    B_CALL _VPutMap
    pop af
    call VPut2DecNum
    ret

.end
END