
; ASTRO version 1.0
;  Benjamin Moody
;  benjamin@ecg.mit,edu

; TI-83+ Flash Application

#include "ti83plus.inc"

#define rOP1ToOP2 08h
#define rFindSym 10h
#define rPushRealO1 18h
#define rMov9ToOP1 20h
#define rFPAdd 30h

    .org $4000

EXT_APP = 1

    .db $80, $0F                        ; Field: Program length
    .db 0, 0, 0, 0                      ; Length = 0

#ifdef SIGN
    .db $80, $13                        ; Field: Program type
                                        ; Type = Developer ID
#else
    .db $80, $12                        ; Field: Program type
    .db $01,$04                         ; Type = Shareware, TI-83+
#endif

    .db $80, $21                        ; Field: App ID
    .db 1                               ; Id = 1
    .db $80, $31                        ; Field: App Build
    .db 1                               ; Build = 1
    .db $80, $48                        ; Field: App Name
    .db "Astro * "                      ; Name
    .db $80, $81                        ; Field: App Pages
    .db 1                               ; App Pages = 1
    .db $80, $90                        ; No default splash screen
    .db $03, $26, $09, $04, $04, $6f, $1b, $80              ; Field: Date stamp- 5/12/1999
    .db $02, $0d, $40, $a1, $6b, $99, $f6, $59, $bc, $67    ; Dummy encrypted TI date stamp signature
    .db $f5, $85, $9c, $09, $6c, $0f, $b4, $03, $9b, $c9
    .db $03, $32, $2c, $e0, $03, $20, $e3, $2c, $f4, $2d 
    .db $73, $b4, $27, $c4, $a0, $72, $54, $b9, $ea, $7c 
    .db $3b, $aa, $16, $f6, $77, $83, $7a, $ee, $1a, $d4 
    .db $42, $4c, $6b, $8b, $13, $1f, $bb, $93, $8b, $fc 
    .db $19, $1c, $3c, $ec, $4d, $e5, $75
    .db $80, $7f                        ; Field: Program Image length
    .db   0, 0, 0, 0                    ; Length = 0, N/A
    .db   0, 0, 0, 0                    ; Reserved
    .db   0, 0, 0, 0                    ; Reserved
    .db   0, 0, 0, 0                    ; Reserved
    .db   0, 0, 0, 0                    ; Reserved

StarDec_f       .equ statVars
StarRA_f        .equ StarDec_f+9

StoredData      .equ StarRA_f+9
ViewLng         .equ StoredData         ; 2     View longitude (DM)
ViewLat         .equ ViewLng+2          ; 2     View latitude (DM)
ViewAzm         .equ ViewLat+2          ; 2     View's azimuth
ViewAlt         .equ ViewAzm+2          ; 2     View's altiude
Zoom            .equ ViewAlt+2          ; 1     zoom
ViewSecond      .equ Zoom+1             ; 1     Second
ViewMinute      .equ ViewSecond+1       ; 1     Minute
ViewHour        .equ ViewMinute+1       ; 1     Hour
ViewMDay        .equ ViewHour+1         ; 1     Day of month
ViewMonth       .equ ViewMDay+1         ; 1     Month
ViewYear        .equ ViewMonth+1        ; 2     Year
ViewTZ          .equ ViewYear+2         ; 1     Time zone
StarDepth       .equ ViewTZ+1           ; 1     Magnitude of dimmest visible star
NumObjs         .equ StarDepth+1           ; 1     Number of objects
ObjectsOn       .equ NumObjs+1          ; 12    Objects activated
DataLoc         .equ ObjectsOn+12       ; 72    Locations of data (8 references, 9 bytes each)
; here options
StoredDataEnd   .equ DataLoc+72

ViewRA_f        .equ StoredDataEnd      ; 9
ViewDec_f       .equ ViewRA_f+9         ; 9

AVpage          .equ ViewDec_f+9        ; 1     Not used yet
AVstart         .equ AVpage+1           ; 2     Variable data

MenuStart       .equ AVstart+2          ; 1     Start of menu for option/object screens
MenuSelect      .equ MenuStart+1        ; 1     Menu selection
MenuSelect2     .equ MenuSelect+1       ; 1
LastKey         .equ MenuSelect2+1      ; 1     Temporary storage for a key value in a menu screen
CursorMode      .equ LastKey+1          ; 1     Flagged if info cursor on
CursorSelect    .equ CursorMode+1       ; 2     Pointing to info selection
ViewAbsDay      .equ CursorSelect+2     ; 9     Absolute day

SpriteMem       .equ ViewAbsDay+9         ; 58    Space to load sprite into RAM

; So far 219 bytes of statvars (531) used

#define ANGLE(d,m) d*256+m
#define ANGLE_M(d,m) d*256+m+128

#define AVLength DefaultAVend-DefaultAV


VMajor          .equ 1
VMinor          .equ 0
Version         .equ VMinor+VMajor*256


;------------------------------------------------
;  Main Program
;------------------------------------------------

Start:
    set trigDeg,(iy+trigFlags)      ; degree mode
    set plotLoc,(iy+plotFlags)      ; don't draw on graph
    res bufferOnly,(iy+plotFlag3)   ; don't draw only on graph
    res indicOnly,(iy+indicFlags)   ; allow keyboard scanning
    res indicRun,(iy+indicFlags)    ; no run indicator
    set apdAble,(iy+apdFlags)       ; turn on apd
    res apdRunning,(iy+apdFlags)    ; pause apd
    set fullScrnDraw,(iy+apiFlg4)  ; allow full screen
    
    ld hl,16                        ; Used during math functions
    B_CALL _AllocFPS                ;   Maybe I don't actually need 16...
    
    call CheckCreateAV              ; If exists, load last session's prefs, otherwise
    call LoadAppVar                 ;    load defaults and create new appvar
;    ld a,8
;    ld (NumObjs),a
    
    xor a
    ld (CursorMode),a               ; Info cursor off by default

    ld hl,Stars-5
    call InfoCursorDown
        
    B_CALL _ForceFullScreen

    
MainScreen:
    B_CALL _ClrLCDFull
    call RedrawScreen
MainLoop:
    B_CALL _ApdSetup                ; Reset APD counter
    B_CALL _GetKey
    res apdRunning,(iy+apdFlags)    ; pause APD until next user input
    cp kQuit
    jp z,QuitKey
    cp kYequ
    jp z,ObjectsScreen
    cp kWindow
    jp z,LocationScreen
    cp kZoom
    jp z,TimeScreen 
    cp kMode
    jp z,OptionsScreen
;    cp kStore
;    jp z,SavePicScreen
;    cp kStat
;    jp z,AboutScreen
    cp kEnter
    call z,ToggleInfoCur
    cp kUp
    call z,InfoCursorUp
    cp kDown
    call z,InfoCursorDown
    jr MainLoop
    
ToggleInfoCur:
    ld a,(CursorMode)
    cpl
    ld (CursorMode),a
    or a
    jr nz,DrawInfoCur
    B_CALL _GrBufCpy
    ret
DrawInfoCur:
    ld hl,(CursorSelect)
    ld c,(hl)   ; Minutes RA
    inc hl
    ld b,(hl)   ; Hours RA
    inc hl
    push hl
    call HMToFloat
    B_CALL _OP1ToOP4
    pop hl
    ld c,(hl)   ; Minutes Decl.
    inc hl
    ld b,(hl)   ; Degrees Decl.
    inc hl
    call DMToFloat
    B_CALL _OP4ToOP2
    call CalcStar
    ld a,2
    call DrawObj
    ret nz

    ld bc,0
    ld (penCol),bc

    ld a, $fd
    ld hl,(CursorSelect)
DInfCurLp:
    dec hl
    cp (hl)
    jr nz,DInfCurLp
    call VPutS
    xor a
    ret

InfoCursorDown:
    B_CALL _GrBufCpy
    ld hl,(CursorSelect)
InfCurDnLp1:
    inc hl
    inc hl
    inc hl
    inc hl
    inc hl    
    ld a,(hl)
    cp $ff
    jr z,InfCurLpDn
    cp $fd
    jr nz,InfCurDnLp1
InfCurDnLp2:
    ld a,(hl)
    inc hl
    or a
    jr nz,InfCurDnLp2
    ld (CursorSelect),hl
    call DrawInfoCur
    jr nz,InfoCursorDown
    ret

InfCurLpDn:
    ld hl,Stars-5
    jr InfCurDnLp1   ; NB this will be stuck in a loop if there are no named stars!

InfoCursorUp:
    B_CALL _GrBufCpy
    ld a, $fd
    ld hl,(CursorSelect)
InfCurUpLp1:
    dec hl
    cp (hl)
    jr nz,InfCurUpLp1
    dec hl
InfCurUpLp2:
    dec hl
    cp (hl)
    jr nz,InfCurUpLp2

    inc hl
    ld a,(hl)
    or a
    jr z,DrawInfoCur
    ld de,0
    ld (penCol),de
    call VPutS
    ld (CursorSelect),hl
    call DrawInfoCur
    jr nz,InfoCursorUp
    ret


QuitKey:
    call SaveAppVar
QuitNow:
    B_CALL _DelRes                  ; because we used statvars
    set apdRunning,(iy+apdFlags)
    B_CALL _ClrScrnFull
    B_CALL _ReloadAppEntryVecs
    B_JUMP _JForceCmdNoChar

;------------------------------------------------
;  System functions
;------------------------------------------------

PutS:
    push bc
    push af
    ld a,(winBtm)
    ld b,a
PutS10:
    ld a,(hl)
    inc hl
    or a
    scf
    jr z,PutS20
    B_CALL _PutC
    ld a,(curRow)
    cp b
    jr c,PutS10
PutS20:
    pop bc
    ld a,b
    pop bc
    ret

VPutS:
    push af
    push de
VPutS10:
    ld a,(hl)
    inc hl
    or a
    jr z,VPutS20
    B_CALL _VPutMap
    jr VPutS10
VPutS20:
    pop de
    pop af
    ret


Mov9ToOP1:
    ld de,OP1
    jr Mov9
Mov9ToOP2:
    ld de,OP2
    jr Mov9
Mov9ToOP4:
    ld de,OP4
    jr Mov9
Mov9ToOP5:
    ld de,OP5
Mov9:
    ldi
    ldi
    ldi
    ldi
    ldi
    ldi
    ldi
    ldi
    ldi
    ret

#include "astrappv.asm"     ; Memory and AppVar
#include "astrdraw.asm"     ; Drawing
#include "astrmath.asm"     ; Math
#include "astrops.asm"     ; Options Screen (rather limited)
#include "astrobjs.asm"     ; Object Selection
#include "astrtime.asm"     ; Time and Date Selection
#include "astrloc.asm"      ; Location Selection
#include "astrdat.asm"      ; Data

.db $ff,$fe,0,$fd,0
Stars:
#include "ybs.asm"        ; Stars Data File

.end
END